#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <map>

#include <Corrade/Containers/StringView.h>

#include <Magnum/Types.h>

using namespace Corrade;
using namespace Containers::Literals;
using namespace Magnum;

enum AccessorySize {
    S,
    M,
    L,
    XL
};

struct AccessoryData{
    Containers::StringView name;
    AccessorySize size = AccessorySize::S;
};

static const std::map<Int, AccessoryData> accessories {
    // region Primitives
    {1,  {"Cube"_s,           AccessorySize::S}},
    {2,  {"Pentagon"_s,       AccessorySize::S}},
    {3,  {"Hexagon"_s,        AccessorySize::S}},
    {4,  {"Cylinder"_s,       AccessorySize::S}},
    {5,  {"Sphere"_s,         AccessorySize::S}},
    {6,  {"TriPyramid"_s,     AccessorySize::S}},
    {7,  {"SquPyramid"_s,     AccessorySize::S}},
    {8,  {"PenPyramid"_s,     AccessorySize::S}},
    {9,  {"HexPyramid"_s,     AccessorySize::S}},
    {10, {"Cone"_s,           AccessorySize::S}},
    {11, {"SquStick"_s,       AccessorySize::S}},
    {12, {"PenStick"_s,       AccessorySize::S}},
    {13, {"HexStick"_s,       AccessorySize::S}},
    {14, {"CycStick"_s,       AccessorySize::S}},
    {15, {"Capsule"_s,        AccessorySize::S}},
    {16, {"Decal Pad 01"_s,   AccessorySize::S}},
    {17, {"Decal Pad 02"_s,   AccessorySize::S}},
    {18, {"Decal Pad 03"_s,   AccessorySize::S}},
    {19, {"Decal Pad 04"_s,   AccessorySize::S}},
    {20, {"Decal Pad 05"_s,   AccessorySize::S}},
    {21, {"Triangle"_s,       AccessorySize::S}},
    {22, {"ThinStar"_s,       AccessorySize::S}},
    {23, {"Star"_s,           AccessorySize::S}},
    {24, {"SixSideStar"_s,    AccessorySize::S}},
    {25, {"Asterisk"_s,       AccessorySize::S}},
    {26, {"Ring"_s,           AccessorySize::S}},
    {27, {"SawedRing"_s,      AccessorySize::S}},
    {28, {"HalfRing"_s,       AccessorySize::S}},
    {29, {"Cresent"_s,        AccessorySize::S}},
    {30, {"Donut"_s,          AccessorySize::S}},
    {31, {"FiveCogWheel"_s,   AccessorySize::S}},
    {32, {"SixCogWheel"_s,    AccessorySize::S}},
    {33, {"SevenCogWheel"_s,  AccessorySize::S}},
    {34, {"EightCogWheel"_s,  AccessorySize::S}},
    {35, {"TwelveCogWheel"_s, AccessorySize::S}},

    {51, {"SquBevel"_s,  AccessorySize::S}},
    {52, {"TriBevel"_s,  AccessorySize::S}},
    {53, {"PenBevel"_s,  AccessorySize::S}},
    {54, {"HexBevel"_s,  AccessorySize::S}},
    {55, {"CycBevel"_s,  AccessorySize::S}},
    {56, {"RecBevel"_s,  AccessorySize::S}},
    {57, {"DaiBevel"_s,  AccessorySize::S}},
    {58, {"MonBevel"_s,  AccessorySize::S}},
    {59, {"CofBevel"_s,  AccessorySize::S}},
    {60, {"JevBevel"_s,  AccessorySize::S}},
    {61, {"SquEmboss"_s, AccessorySize::S}},
    {62, {"TriEmboss"_s, AccessorySize::S}},
    {63, {"PenEmboss"_s, AccessorySize::S}},
    {64, {"HexEmboss"_s, AccessorySize::S}},
    {65, {"CycEmboss"_s, AccessorySize::S}},
    {66, {"RecEmboss"_s, AccessorySize::S}},
    {67, {"DaiEmboss"_s, AccessorySize::S}},
    {68, {"MonEmboss"_s, AccessorySize::S}},
    {69, {"CofEmboss"_s, AccessorySize::S}},
    {70, {"JevEmboss"_s, AccessorySize::S}},

    {101, {"Flat Hex Pin"_s,      AccessorySize::S}},
    {102, {"Cross Circle Pin"_s,  AccessorySize::S}},
    {103, {"Flat Circle Pin"_s,   AccessorySize::S}},
    {104, {"Hex Circle Pin"_s,    AccessorySize::S}},
    {105, {"Circle Button Pin"_s, AccessorySize::S}},
    {106, {"Hexagon Pin"_s,       AccessorySize::S}},
    {107, {"Cross Square Pin"_s,  AccessorySize::S}},
    {108, {"Flat Square Pin"_s,   AccessorySize::S}},
    {109, {"Quad Corner Pin"_s,   AccessorySize::S}},
    {110, {"Bi Corner Pin"_s,     AccessorySize::S}},
    {111, {"Circle Pin"_s,        AccessorySize::S}},
    {112, {"Flat End Pin"_s,      AccessorySize::S}},
    {113, {"Flat Cut Pin"_s,      AccessorySize::S}},
    {114, {"Radial Pin"_s,        AccessorySize::S}},
    {115, {"Diamiter Pin"_s,      AccessorySize::S}},

    {151, {"TriPoint"_s,          AccessorySize::S}},
    {152, {"SquPoint"_s,          AccessorySize::S}},
    {153, {"PenPoint"_s,          AccessorySize::S}},
    {154, {"HexPoint"_s,          AccessorySize::S}},
    {155, {"CycPoint"_s,          AccessorySize::S}},
    {156, {"Bevel SquCutPoint"_s, AccessorySize::S}},
    {157, {"Bevel HexCutPoint"_s, AccessorySize::S}},
    {158, {"Bevel HexPoint"_s,    AccessorySize::S}},
    {159, {"Bevel CycCutPoint"_s, AccessorySize::S}},
    {160, {"Bevel CycPoint"_s,    AccessorySize::S}},

    {201, {"Shaped Edge 01"_s, AccessorySize::M}},
    {202, {"Shaped Edge 02"_s, AccessorySize::M}},
    {203, {"Shaped Edge 03"_s, AccessorySize::M}},
    {204, {"Shaped Edge 04"_s, AccessorySize::M}},
    {205, {"Shaped Edge 05"_s, AccessorySize::M}},
    {206, {"Shaped Edge 06"_s, AccessorySize::M}},
    {207, {"Shaped Edge 07"_s, AccessorySize::M}},
    {208, {"Shaped Edge 08"_s, AccessorySize::M}},
    {209, {"Shaped Edge 09"_s, AccessorySize::M}},
    {210, {"Shaped Edge 10"_s, AccessorySize::M}},
    {211, {"Shaped Edge 11"_s, AccessorySize::M}},
    {212, {"Shaped Edge 12"_s, AccessorySize::M}},
    {213, {"Shaped Edge 13"_s, AccessorySize::M}},
    {214, {"Shaped Edge 14"_s, AccessorySize::M}},
    {215, {"Shaped Edge 15"_s, AccessorySize::M}},
    {216, {"Shaped Edge 16"_s, AccessorySize::M}},
    {217, {"Shaped Edge 17"_s, AccessorySize::M}},
    {218, {"Shaped Edge 18"_s, AccessorySize::M}},
    {219, {"Shaped Edge 19"_s, AccessorySize::M}},
    {220, {"Shaped Edge 20"_s, AccessorySize::M}},

    {251, {"Fish Tail 01"_s,       AccessorySize::M}},
    {252, {"Fish Tail 02"_s,       AccessorySize::M}},
    {253, {"Fish Tail 03"_s,       AccessorySize::M}},
    {254, {"Fish Tail 04"_s,       AccessorySize::M}},
    {255, {"Fish Tail 05"_s,       AccessorySize::M}},
    {256, {"Based Separator 01"_s, AccessorySize::M}},
    {257, {"Based Separator 02"_s, AccessorySize::M}},
    {258, {"Based Separator 03"_s, AccessorySize::M}},
    {259, {"Based Separator 04"_s, AccessorySize::M}},
    {260, {"Based Separator 05"_s, AccessorySize::M}},
    {261, {"Based Separator 06"_s, AccessorySize::M}},
    {262, {"Based Separator 07"_s, AccessorySize::M}},
    {263, {"Based Separator 08"_s, AccessorySize::M}},
    {264, {"Based Separator 09"_s, AccessorySize::M}},
    {265, {"Based Separator 10"_s, AccessorySize::M}},

    {301, {"Rectangular Box 01"_s, AccessorySize::M}},
    {302, {"Rectangular Box 02"_s, AccessorySize::M}},
    {303, {"Rectangular Box 03"_s, AccessorySize::M}},
    {304, {"Rectangular Box 04"_s, AccessorySize::M}},
    {305, {"Rectangular Box 05"_s, AccessorySize::M}},
    {306, {"CofBox 01"_s,          AccessorySize::M}},
    {307, {"CofBox 02"_s,          AccessorySize::M}},
    {308, {"CofBox 03"_s,          AccessorySize::M}},
    {309, {"CofBox 04"_s,          AccessorySize::M}},
    {310, {"CofBox 05"_s,          AccessorySize::M}},
    {311, {"Triangular Box 01"_s,  AccessorySize::M}},
    {312, {"Triangular Box 02"_s,  AccessorySize::M}},
    {313, {"Triangular Box 03"_s,  AccessorySize::M}},
    {314, {"Triangular Box 04"_s,  AccessorySize::M}},
    {315, {"Triangular Box 05"_s,  AccessorySize::M}},
    {316, {"Diagonal Box A01"_s,   AccessorySize::M}},
    {317, {"Diagonal Box A02"_s,   AccessorySize::M}},
    {318, {"Diagonal Box A03"_s,   AccessorySize::M}},
    {319, {"Diagonal Box A04"_s,   AccessorySize::M}},
    {320, {"Diagonal Box A05"_s,   AccessorySize::M}},
    {321, {"Diagonal Box B01"_s,   AccessorySize::M}},
    {322, {"Diagonal Box B02"_s,   AccessorySize::M}},
    {323, {"Diagonal Box B03"_s,   AccessorySize::M}},
    {324, {"Diagonal Box B04"_s,   AccessorySize::M}},
    {325, {"Diagonal Box B05"_s,   AccessorySize::M}},
    // endregion

    // region Armours
    {1001, {"Short Layer 01"_s,         AccessorySize::M}},
    {1002, {"Short Layer 02"_s,         AccessorySize::M}},
    {1003, {"Short Layer 03"_s,         AccessorySize::M}},
    {1004, {"Short Layer 04"_s,         AccessorySize::M}},
    {1005, {"Short Layer 05"_s,         AccessorySize::M}},
    {1006, {"Long Layer 01"_s,          AccessorySize::M}},
    {1007, {"Long Layer 02"_s,          AccessorySize::M}},
    {1008, {"Long Layer 03"_s,          AccessorySize::M}},
    {1009, {"Long Layer 04"_s,          AccessorySize::M}},
    {1010, {"Long Layer 05"_s,          AccessorySize::M}},
    {1011, {"Diagonal Long Layer 01"_s, AccessorySize::M}},
    {1012, {"Diagonal Long Layer 02"_s, AccessorySize::M}},
    {1013, {"Diagonal Long Layer 03"_s, AccessorySize::M}},
    {1014, {"Diagonal Long Layer 04"_s, AccessorySize::M}},
    {1015, {"Diagonal Long Layer 05"_s, AccessorySize::M}},

    {1051, {"Sloped Layer 01"_s, AccessorySize::M}},
    {1052, {"Sloped Layer 02"_s, AccessorySize::M}},
    {1053, {"Sloped Layer 03"_s, AccessorySize::M}},
    {1054, {"Sloped Layer 04"_s, AccessorySize::M}},
    {1055, {"Sloped Layer 05"_s, AccessorySize::M}},
    {1056, {"Sloped Layer 06"_s, AccessorySize::M}},
    {1057, {"Sloped Layer 07"_s, AccessorySize::M}},
    {1058, {"Sloped Layer 08"_s, AccessorySize::M}},
    {1059, {"Sloped Layer 09"_s, AccessorySize::M}},
    {1060, {"Sloped Layer 10"_s, AccessorySize::M}},
    {1061, {"Sloped Layer 11"_s, AccessorySize::M}},
    {1062, {"Sloped Layer 12"_s, AccessorySize::M}},
    {1063, {"Sloped Layer 13"_s, AccessorySize::M}},
    {1064, {"Sloped Layer 14"_s, AccessorySize::M}},
    {1065, {"Sloped Layer 15"_s, AccessorySize::M}},

    {1101, {"Raised Center 01"_s, AccessorySize::M}},
    {1102, {"Raised Center 02"_s, AccessorySize::M}},
    {1103, {"Raised Center 03"_s, AccessorySize::M}},
    {1104, {"Raised Center 04"_s, AccessorySize::M}},
    {1105, {"Raised Center 05"_s, AccessorySize::M}},
    {1106, {"Raised Block 01"_s,  AccessorySize::M}},
    {1107, {"Raised Block 02"_s,  AccessorySize::M}},
    {1108, {"Raised Block 03"_s,  AccessorySize::M}},
    {1109, {"Raised Pointed"_s,   AccessorySize::M}},
    {1110, {"Raised Cover"_s,     AccessorySize::M}},
    {1111, {"Raised Slant 01"_s,  AccessorySize::M}},
    {1112, {"Raised Slant 02"_s,  AccessorySize::M}},
    {1113, {"Raised Slant 03"_s,  AccessorySize::M}},
    {1114, {"Raised Slant 04"_s,  AccessorySize::M}},
    {1115, {"Raised Slant 05"_s,  AccessorySize::M}},

    {1151, {"Wide Patch 01"_s, AccessorySize::L}},
    {1152, {"Wide Patch 02"_s, AccessorySize::L}},
    {1153, {"Wide Patch 03"_s, AccessorySize::L}},
    {1154, {"Wide Patch 04"_s, AccessorySize::L}},
    {1155, {"Wide Patch 05"_s, AccessorySize::L}},

    {1201, {"Pointed Armour 01"_s, AccessorySize::L}},
    {1202, {"Pointed Armour 02"_s, AccessorySize::L}},
    {1203, {"Pointed Armour 03"_s, AccessorySize::L}},
    {1204, {"Pointed Armour 04"_s, AccessorySize::L}},
    {1205, {"Pointed Armour 05"_s, AccessorySize::L}},
    {1206, {"Pointed Armour 06"_s, AccessorySize::L}},
    {1207, {"Pointed Armour 07"_s, AccessorySize::L}},
    {1208, {"Pointed Armour 08"_s, AccessorySize::L}},
    {1209, {"Pointed Armour 09"_s, AccessorySize::L}},
    {1210, {"Pointed Armour 10"_s, AccessorySize::L}},
    {1211, {"Pointed Armour 11"_s, AccessorySize::L}},
    {1212, {"Pointed Armour 12"_s, AccessorySize::L}},
    {1213, {"Pointed Armour 13"_s, AccessorySize::L}},
    {1214, {"Pointed Armour 14"_s, AccessorySize::L}},
    {1215, {"Pointed Armour 15"_s, AccessorySize::L}},

    {1251, {"E Limb Cover 01"_s, AccessorySize::L}},
    {1252, {"E Limb Cover 02"_s, AccessorySize::L}},
    {1253, {"E Limb Cover 03"_s, AccessorySize::L}},
    {1254, {"E Limb Cover 04"_s, AccessorySize::L}},
    {1255, {"E Limb Cover 05"_s, AccessorySize::L}},
    {1256, {"E Limb Cover 06"_s, AccessorySize::L}},
    {1257, {"E Limb Cover 07"_s, AccessorySize::L}},
    {1258, {"E Limb Cover 08"_s, AccessorySize::L}},
    {1259, {"E Limb Cover 09"_s, AccessorySize::L}},
    {1260, {"E Limb Cover 10"_s, AccessorySize::L}},

    {1301, {"C Limb Cover 01"_s, AccessorySize::L}},
    {1302, {"C Limb Cover 02"_s, AccessorySize::L}},
    {1303, {"C Limb Cover 03"_s, AccessorySize::L}},
    {1304, {"C Limb Cover 04"_s, AccessorySize::L}},
    {1305, {"C Limb Cover 05"_s, AccessorySize::L}},
    {1306, {"C Limb Cover 06"_s, AccessorySize::L}},
    {1307, {"C Limb Cover 07"_s, AccessorySize::L}},
    {1308, {"C Limb Cover 08"_s, AccessorySize::L}},
    {1309, {"C Limb Cover 09"_s, AccessorySize::L}},
    {1310, {"C Limb Cover 10"_s, AccessorySize::L}},
    {1311, {"C Limb Cover 11"_s, AccessorySize::L}},
    {1312, {"C Limb Cover 12"_s, AccessorySize::L}},
    {1313, {"C Limb Cover 13"_s, AccessorySize::L}},
    {1314, {"C Limb Cover 14"_s, AccessorySize::L}},
    {1315, {"C Limb Cover 15"_s, AccessorySize::L}},
    {1316, {"C Limb Cover 16"_s, AccessorySize::L}},
    {1317, {"C Limb Cover 17"_s, AccessorySize::L}},
    {1318, {"C Limb Cover 18"_s, AccessorySize::L}},
    {1319, {"C Limb Cover 19"_s, AccessorySize::L}},
    {1320, {"C Limb Cover 20"_s, AccessorySize::L}},

    {1351, {"P Limb Cover 01"_s, AccessorySize::XL}},
    {1352, {"P Limb Cover 02"_s, AccessorySize::XL}},
    {1353, {"P Limb Cover 03"_s, AccessorySize::XL}},
    {1354, {"P Limb Cover 04"_s, AccessorySize::XL}},
    {1355, {"P Limb Cover 05"_s, AccessorySize::XL}},

    {1401, {"Flat Cover 01"_s, AccessorySize::XL}},
    {1402, {"Flat Cover 02"_s, AccessorySize::XL}},
    {1403, {"Flat Cover 03"_s, AccessorySize::XL}},
    {1404, {"Flat Cover 04"_s, AccessorySize::XL}},
    {1405, {"Flat Cover 05"_s, AccessorySize::XL}},
    {1406, {"Flat Cover 06"_s, AccessorySize::XL}},
    {1407, {"Flat Cover 07"_s, AccessorySize::XL}},
    {1408, {"Flat Cover 08"_s, AccessorySize::XL}},
    {1409, {"Flat Cover 09"_s, AccessorySize::XL}},
    {1410, {"Flat Cover 10"_s, AccessorySize::XL}},

    {1451, {"L Side Opening 01"_s, AccessorySize::XL}},
    {1452, {"L Side Opening 02"_s, AccessorySize::XL}},
    {1453, {"L Side Opening 03"_s, AccessorySize::XL}},
    {1454, {"L Side Opening 04"_s, AccessorySize::XL}},
    {1455, {"L Side Opening 05"_s, AccessorySize::XL}},
    {1456, {"L Side Opening 06"_s, AccessorySize::XL}},
    {1457, {"L Side Opening 07"_s, AccessorySize::XL}},
    {1458, {"L Side Opening 08"_s, AccessorySize::XL}},
    {1459, {"L Side Opening 09"_s, AccessorySize::XL}},
    {1460, {"L Side Opening 10"_s, AccessorySize::XL}},
    // endregion

    // region Components
    {2001, {"Disc Padding 01"_s,  AccessorySize::M}},
    {2002, {"Disc Padding 02"_s,  AccessorySize::M}},
    {2003, {"Disc Padding 03"_s,  AccessorySize::M}},
    {2004, {"Disc Padding 04"_s,  AccessorySize::M}},
    {2005, {"Disc Padding 05"_s,  AccessorySize::M}},
    {2006, {"Thin Padding 01"_s,  AccessorySize::M}},
    {2007, {"Thin Padding 02"_s,  AccessorySize::M}},
    {2008, {"Thin Padding 03"_s,  AccessorySize::M}},
    {2009, {"Thin Padding 04"_s,  AccessorySize::M}},
    {2010, {"Thin Padding 05"_s,  AccessorySize::M}},
    {2011, {"Thick Padding 01"_s, AccessorySize::M}},
    {2012, {"Thick Padding 02"_s, AccessorySize::M}},
    {2013, {"Thick Padding 03"_s, AccessorySize::M}},
    {2014, {"Thick Padding 04"_s, AccessorySize::M}},
    {2015, {"Thick Padding 05"_s, AccessorySize::M}},
    {2016, {"Thick Padding 06"_s, AccessorySize::M}},
    {2017, {"Thick Padding 07"_s, AccessorySize::M}},
    {2018, {"Thick Padding 08"_s, AccessorySize::M}},
    {2019, {"Thick Padding 09"_s, AccessorySize::M}},
    {2020, {"Thick Padding 10"_s, AccessorySize::M}},
    {2021, {"CSide Padding 01"_s, AccessorySize::M}},
    {2022, {"CSide Padding 02"_s, AccessorySize::M}},
    {2023, {"CSide Padding 03"_s, AccessorySize::M}},
    {2024, {"CSide Padding 04"_s, AccessorySize::M}},
    {2025, {"CSide Padding 05"_s, AccessorySize::M}},

    {2051, {"Container 01"_s, AccessorySize::L}},
    {2052, {"Container 02"_s, AccessorySize::L}},
    {2053, {"Container 03"_s, AccessorySize::L}},
    {2054, {"Container 04"_s, AccessorySize::L}},
    {2055, {"Container 05"_s, AccessorySize::L}},

    {2101, {"Plating 01"_s, AccessorySize::L}},
    {2102, {"Plating 02"_s, AccessorySize::L}},
    {2103, {"Plating 03"_s, AccessorySize::L}},
    {2104, {"Plating 04"_s, AccessorySize::L}},
    {2105, {"Plating 05"_s, AccessorySize::L}},

    {2151, {"Complex Base 01"_s, AccessorySize::L}},
    {2152, {"Complex Base 02"_s, AccessorySize::L}},
    {2153, {"Complex Base 03"_s, AccessorySize::L}},
    {2154, {"Complex Base 04"_s, AccessorySize::L}},
    {2155, {"Complex Base 05"_s, AccessorySize::L}},
    {2156, {"Complex Base 06"_s, AccessorySize::L}},
    {2157, {"Complex Base 07"_s, AccessorySize::L}},
    {2158, {"Complex Base 08"_s, AccessorySize::L}},
    {2159, {"Complex Base 09"_s, AccessorySize::L}},
    {2160, {"Complex Base 10"_s, AccessorySize::L}},

    {2201, {"Long Base 01"_s, AccessorySize::XL}},
    {2202, {"Long Base 02"_s, AccessorySize::XL}},
    {2203, {"Long Base 03"_s, AccessorySize::XL}},
    {2204, {"Long Base 04"_s, AccessorySize::XL}},
    {2205, {"Long Base 05"_s, AccessorySize::XL}},

    {2251, {"Straight Wing 01"_s, AccessorySize::XL}},
    {2252, {"Straight Wing 02"_s, AccessorySize::XL}},
    {2253, {"Straight Wing 03"_s, AccessorySize::XL}},
    {2254, {"Straight Wing 04"_s, AccessorySize::XL}},
    {2255, {"Straight Wing 05"_s, AccessorySize::XL}},
    {2256, {"Straight Wing 06"_s, AccessorySize::XL}},
    {2257, {"Straight Wing 07"_s, AccessorySize::XL}},
    {2258, {"Straight Wing 08"_s, AccessorySize::XL}},
    {2259, {"Straight Wing 09"_s, AccessorySize::XL}},
    {2260, {"Straight Wing 10"_s, AccessorySize::XL}},

    {2301, {"Triangular Wing 01"_s, AccessorySize::XL}},
    {2302, {"Triangular Wing 02"_s, AccessorySize::XL}},
    {2303, {"Triangular Wing 03"_s, AccessorySize::XL}},
    {2304, {"Triangular Wing 04"_s, AccessorySize::XL}},
    {2305, {"Triangular Wing 05"_s, AccessorySize::XL}},
    {2306, {"Triangular Wing 06"_s, AccessorySize::XL}},
    {2307, {"Triangular Wing 07"_s, AccessorySize::XL}},
    {2308, {"Triangular Wing 08"_s, AccessorySize::XL}},
    {2309, {"Triangular Wing 09"_s, AccessorySize::XL}},
    {2310, {"Triangular Wing 10"_s, AccessorySize::XL}},
    {2311, {"Triangular Wing 11"_s, AccessorySize::L}},
    {2312, {"Triangular Wing 12"_s, AccessorySize::L}},
    {2313, {"Triangular Wing 13"_s, AccessorySize::L}},
    {2314, {"Triangular Wing 14"_s, AccessorySize::L}},
    {2315, {"Triangular Wing 15"_s, AccessorySize::L}},

    {2351, {"Complex Wing 01"_s, AccessorySize::XL}},
    {2352, {"Complex Wing 02"_s, AccessorySize::XL}},
    {2353, {"Complex Wing 03"_s, AccessorySize::XL}},
    {2354, {"Complex Wing 04"_s, AccessorySize::XL}},
    {2355, {"Complex Wing 05"_s, AccessorySize::XL}},
    {2356, {"Complex Wing 06"_s, AccessorySize::L}},
    {2357, {"Complex Wing 07"_s, AccessorySize::L}},
    {2358, {"Complex Wing 08"_s, AccessorySize::L}},
    {2359, {"Complex Wing 09"_s, AccessorySize::L}},
    {2360, {"Complex Wing 10"_s, AccessorySize::L}},

    {2401, {"Blade 01"_s, AccessorySize::XL}},
    {2402, {"Blade 02"_s, AccessorySize::XL}},
    {2403, {"Blade 03"_s, AccessorySize::XL}},
    {2404, {"Blade 04"_s, AccessorySize::XL}},
    {2405, {"Blade 05"_s, AccessorySize::XL}},
    {2406, {"Blade 06"_s, AccessorySize::XL}},
    {2407, {"Blade 07"_s, AccessorySize::XL}},
    {2408, {"Blade 08"_s, AccessorySize::XL}},
    {2409, {"Blade 09"_s, AccessorySize::XL}},
    {2410, {"Blade 10"_s, AccessorySize::XL}},
    {2411, {"Blade 11"_s, AccessorySize::XL}},
    {2412, {"Blade 12"_s, AccessorySize::XL}},
    {2413, {"Blade 13"_s, AccessorySize::XL}},
    {2414, {"Blade 14"_s, AccessorySize::XL}},
    {2415, {"Blade 15"_s, AccessorySize::XL}},

    {2426, {"Curved Blade 01"_s, AccessorySize::XL}},
    {2427, {"Curved Blade 02"_s, AccessorySize::XL}},
    {2428, {"Curved Blade 03"_s, AccessorySize::XL}},
    {2429, {"Curved Blade 04"_s, AccessorySize::XL}},
    {2430, {"Curved Blade 05"_s, AccessorySize::XL}},
    {2431, {"Axe Head 01"_s,     AccessorySize::XL}},
    {2432, {"Axe Head 02"_s,     AccessorySize::XL}},
    {2433, {"Axe Head 03"_s,     AccessorySize::XL}},
    {2434, {"Axe Head 04"_s,     AccessorySize::XL}},
    {2435, {"Axe Head 05"_s,     AccessorySize::XL}},

    {2451, {"Horn 01"_s, AccessorySize::M}},
    {2452, {"Horn 02"_s, AccessorySize::M}},
    {2453, {"Horn 03"_s, AccessorySize::M}},
    {2454, {"Horn 04"_s, AccessorySize::M}},
    {2455, {"Horn 05"_s, AccessorySize::M}},
    {2456, {"Horn 06"_s, AccessorySize::M}},
    {2457, {"Horn 07"_s, AccessorySize::M}},
    {2458, {"Horn 08"_s, AccessorySize::M}},
    {2459, {"Horn 09"_s, AccessorySize::M}},
    {2460, {"Horn 10"_s, AccessorySize::M}},
    {2461, {"Horn 11"_s, AccessorySize::M}},
    {2462, {"Horn 12"_s, AccessorySize::M}},
    {2463, {"Horn 13"_s, AccessorySize::M}},
    {2464, {"Horn 14"_s, AccessorySize::M}},
    {2465, {"Horn 15"_s, AccessorySize::M}},

    {2471, {"Mask"_s,     AccessorySize::M}},
    {2472, {"Droplet"_s,  AccessorySize::M}},
    {2473, {"Thigh"_s,    AccessorySize::M}},
    {2474, {"LegS"_s,     AccessorySize::M}},
    {2475, {"LegTH"_s,    AccessorySize::M}},
    {2476, {"Plume 01"_s, AccessorySize::M}},
    {2477, {"Plume 02"_s, AccessorySize::M}},
    {2478, {"Plume 03"_s, AccessorySize::M}},
    {2479, {"Plume 04"_s, AccessorySize::M}},
    {2480, {"Plume 05"_s, AccessorySize::M}},

    {2491, {"Tail 01"_s, AccessorySize::XL}},
    {2492, {"Tail 02"_s, AccessorySize::XL}},
    {2493, {"Tail 03"_s, AccessorySize::XL}},
    {2494, {"Tail 04"_s, AccessorySize::XL}},
    {2495, {"Tail 05"_s, AccessorySize::XL}},

    {2501, {"Finger 01"_s, AccessorySize::M}},
    {2502, {"Finger 02"_s, AccessorySize::M}},
    {2503, {"Finger 03"_s, AccessorySize::M}},
    {2504, {"Finger 04"_s, AccessorySize::M}},
    {2505, {"Finger 05"_s, AccessorySize::M}},

    {2510, {"Clamp 01"_s, AccessorySize::M}},
    {2511, {"Clamp 02"_s, AccessorySize::M}},
    {2512, {"Clamp 03"_s, AccessorySize::M}},
    {2513, {"Clamp 04"_s, AccessorySize::M}},
    {2514, {"Clamp 05"_s, AccessorySize::M}},

    {2521, {"Fabric 01"_s, AccessorySize::XL}},
    {2522, {"Fabric 02"_s, AccessorySize::XL}},
    {2523, {"Fabric 03"_s, AccessorySize::XL}},
    {2524, {"Fabric 04"_s, AccessorySize::XL}},
    {2525, {"Fabric 05"_s, AccessorySize::XL}},

    {2551, {"Energy Barrel 01"_s, AccessorySize::XL}},
    {2552, {"Energy Barrel 02"_s, AccessorySize::XL}},
    {2553, {"Energy Barrel 03"_s, AccessorySize::XL}},
    {2554, {"Energy Barrel 04"_s, AccessorySize::XL}},
    {2555, {"Energy Barrel 05"_s, AccessorySize::XL}},

    {2601, {"L Bullet Barrel 01"_s, AccessorySize::XL}},
    {2602, {"L Bullet Barrel 02"_s, AccessorySize::XL}},
    {2603, {"L Bullet Barrel 03"_s, AccessorySize::XL}},
    {2604, {"L Bullet Barrel 04"_s, AccessorySize::XL}},
    {2605, {"L Bullet Barrel 05"_s, AccessorySize::XL}},
    {2606, {"S Bullet Barrel 01"_s, AccessorySize::XL}},
    {2607, {"S Bullet Barrel 02"_s, AccessorySize::XL}},
    {2608, {"S Bullet Barrel 03"_s, AccessorySize::XL}},
    {2609, {"S Bullet Barrel 04"_s, AccessorySize::XL}},
    {2610, {"S Bullet Barrel 05"_s, AccessorySize::XL}},
    {2611, {"B Bullet Barrel 01"_s, AccessorySize::XL}},
    {2612, {"B Bullet Barrel 02"_s, AccessorySize::XL}},
    {2613, {"B Bullet Barrel 03"_s, AccessorySize::XL}},
    {2614, {"B Bullet Barrel 04"_s, AccessorySize::XL}},
    {2615, {"B Bullet Barrel 05"_s, AccessorySize::XL}},
    {2616, {"B Bullet Barrel 06"_s, AccessorySize::XL}},
    {2617, {"B Bullet Barrel 07"_s, AccessorySize::XL}},
    {2618, {"B Bullet Barrel 08"_s, AccessorySize::XL}},
    {2619, {"B Bullet Barrel 09"_s, AccessorySize::XL}},
    {2620, {"B Bullet Barrel 10"_s, AccessorySize::XL}},

    {2651, {"Cylinder Scope 01"_s, AccessorySize::M}},
    {2652, {"Cylinder Scope 02"_s, AccessorySize::M}},
    {2653, {"Cylinder Scope 03"_s, AccessorySize::M}},
    {2654, {"Cylinder Scope 04"_s, AccessorySize::M}},
    {2655, {"Cylinder Scope 05"_s, AccessorySize::M}},
    {2656, {"Elec Scope 01"_s,     AccessorySize::M}},
    {2657, {"Elec Scope 02"_s,     AccessorySize::M}},
    {2658, {"Elec Scope 03"_s,     AccessorySize::M}},
    {2659, {"Elec Scope 04"_s,     AccessorySize::M}},
    {2660, {"Elec Scope 05"_s,     AccessorySize::M}},
    {2661, {"Mark Scope 01"_s,     AccessorySize::S}},
    {2662, {"Mark Scope 02"_s,     AccessorySize::S}},
    {2663, {"Mark Scope 03"_s,     AccessorySize::S}},
    {2664, {"Mark Scope 04"_s,     AccessorySize::S}},
    {2665, {"Mark Scope 05"_s,     AccessorySize::S}},

    {2701, {"S Single Weaponry"_s,      AccessorySize::M}},
    {2702, {"S Packed Weaponry 01"_s,   AccessorySize::M}},
    {2703, {"S Packed Weaponry 02"_s,   AccessorySize::M}},
    {2704, {"S Packed Weaponry 03"_s,   AccessorySize::M}},
    {2705, {"S Packed Weaponry 04"_s,   AccessorySize::M}},
    {2706, {"L Single Weaponry"_s,      AccessorySize::XL}},
    {2707, {"L Packed Weaponry 01"_s,   AccessorySize::XL}},
    {2708, {"L Packed Weaponry 02"_s,   AccessorySize::XL}},
    {2709, {"L Packed Weaponry 03"_s,   AccessorySize::XL}},
    {2710, {"L Packed Weaponry 04"_s,   AccessorySize::XL}},
    {2711, {"Atk Single Weaponry"_s,    AccessorySize::XL}},
    {2712, {"Atk Packed Weaponry 01"_s, AccessorySize::XL}},
    {2713, {"Atk Packed Weaponry 02"_s, AccessorySize::XL}},
    {2714, {"Atk Packed Weaponry 03"_s, AccessorySize::XL}},
    {2715, {"Atk Packed Weaponry 04"_s, AccessorySize::XL}},

    {2751, {"Vent 01"_s, AccessorySize::M}},
    {2752, {"Vent 02"_s, AccessorySize::M}},
    {2753, {"Vent 03"_s, AccessorySize::M}},
    {2754, {"Vent 04"_s, AccessorySize::M}},
    {2755, {"Vent 05"_s, AccessorySize::M}},
    {2756, {"Vent 06"_s, AccessorySize::M}},
    {2757, {"Vent 07"_s, AccessorySize::M}},
    {2758, {"Vent 08"_s, AccessorySize::M}},
    {2759, {"Vent 09"_s, AccessorySize::M}},
    {2760, {"Vent 10"_s, AccessorySize::M}},

    {2901, {"Complex Construct 01"_s, AccessorySize::L}},
    {2902, {"Complex Construct 02"_s, AccessorySize::L}},
    {2903, {"Complex Construct 03"_s, AccessorySize::L}},
    {2904, {"Complex Construct 04"_s, AccessorySize::L}},
    {2905, {"Complex Construct 05"_s, AccessorySize::L}},

    {2950, {"Q Mask 01"_s, AccessorySize::M}},
    {2951, {"Q Mask 02"_s, AccessorySize::M}},
    {2952, {"Q Mask 03"_s, AccessorySize::M}},
    {2953, {"Q Mask 04"_s, AccessorySize::M}},
    {2954, {"Q Mask 05"_s, AccessorySize::M}},
    // endregion

    // region Connectors
    {3001, {"Circular Vent 01"_s, AccessorySize::M}},
    {3002, {"Circular Vent 02"_s, AccessorySize::M}},
    {3003, {"Circular Vent 03"_s, AccessorySize::M}},
    {3004, {"Circular Vent 04"_s, AccessorySize::M}},
    {3005, {"Circular Vent 05"_s, AccessorySize::M}},
    {3006, {"Circular Vent 06"_s, AccessorySize::M}},
    {3007, {"Circular Vent 07"_s, AccessorySize::M}},
    {3008, {"Circular Vent 08"_s, AccessorySize::M}},
    {3009, {"Circular Vent 09"_s, AccessorySize::M}},
    {3010, {"Circular Vent 10"_s, AccessorySize::M}},
    {3011, {"Circular Vent 11"_s, AccessorySize::M}},
    {3012, {"Circular Vent 12"_s, AccessorySize::M}},
    {3013, {"Circular Vent 13"_s, AccessorySize::M}},
    {3014, {"Circular Vent 14"_s, AccessorySize::M}},
    {3015, {"Circular Vent 15"_s, AccessorySize::M}},

    {3051, {"Reactor 01"_s, AccessorySize::L}},
    {3052, {"Reactor 02"_s, AccessorySize::L}},
    {3053, {"Reactor 03"_s, AccessorySize::L}},
    {3054, {"Reactor 04"_s, AccessorySize::L}},
    {3055, {"Reactor 05"_s, AccessorySize::L}},

    {3101, {"Connecting Tube 01"_s, AccessorySize::XL}},
    {3102, {"Connecting Tube 02"_s, AccessorySize::XL}},
    {3103, {"Connecting Tube 03"_s, AccessorySize::XL}},
    {3104, {"Connecting Tube 04"_s, AccessorySize::XL}},
    {3105, {"Connecting Tube 05"_s, AccessorySize::XL}},

    {3151, {"Latch 01"_s, AccessorySize::M}},
    {3152, {"Latch 02"_s, AccessorySize::M}},
    {3153, {"Latch 03"_s, AccessorySize::M}},
    {3154, {"Latch 04"_s, AccessorySize::M}},
    {3155, {"Latch 05"_s, AccessorySize::M}},
    {3156, {"Latch 06"_s, AccessorySize::M}},
    {3157, {"Latch 07"_s, AccessorySize::M}},
    {3158, {"Latch 08"_s, AccessorySize::M}},
    {3159, {"Latch 09"_s, AccessorySize::M}},
    {3160, {"Latch 10"_s, AccessorySize::M}},
    {3161, {"Latch 11"_s, AccessorySize::M}},
    {3162, {"Latch 12"_s, AccessorySize::M}},
    {3163, {"Latch 13"_s, AccessorySize::M}},
    {3164, {"Latch 14"_s, AccessorySize::M}},
    {3165, {"Latch 15"_s, AccessorySize::M}},

    {3201, {"Short Connector 01"_s, AccessorySize::M}},
    {3202, {"Short Connector 02"_s, AccessorySize::M}},
    {3203, {"Short Connector 03"_s, AccessorySize::M}},
    {3204, {"Short Connector 04"_s, AccessorySize::M}},
    {3205, {"Short Connector 05"_s, AccessorySize::M}},
    {3206, {"Antenna 01"_s,         AccessorySize::S}},
    {3207, {"Antenna 02"_s,         AccessorySize::S}},
    {3208, {"Antenna 03"_s,         AccessorySize::S}},
    {3209, {"Antenna 04"_s,         AccessorySize::S}},
    {3210, {"Antenna 05"_s,         AccessorySize::S}},

    {3226, {"Long Connector 01"_s, AccessorySize::XL}},
    {3227, {"Long Connector 02"_s, AccessorySize::XL}},
    {3228, {"Long Connector 03"_s, AccessorySize::XL}},
    {3229, {"Long Connector 04"_s, AccessorySize::XL}},
    {3230, {"Long Connector 05"_s, AccessorySize::XL}},
    {3231, {"Long Connector 06"_s, AccessorySize::XL}},
    {3232, {"Long Connector 07"_s, AccessorySize::XL}},
    {3233, {"Long Connector 08"_s, AccessorySize::XL}},
    {3234, {"Long Connector 09"_s, AccessorySize::XL}},
    {3235, {"Long Connector 10"_s, AccessorySize::XL}},

    {3251, {"Complex Connector 01"_s, AccessorySize::XL}},
    {3252, {"Complex Connector 02"_s, AccessorySize::XL}},
    {3253, {"Complex Connector 03"_s, AccessorySize::XL}},
    {3254, {"Complex Connector 04"_s, AccessorySize::XL}},
    {3255, {"Complex Connector 05"_s, AccessorySize::XL}},

    {3301, {"Tube Line 01"_s, AccessorySize::L}},
    {3302, {"Tube Line 02"_s, AccessorySize::L}},
    {3303, {"Tube Line 03"_s, AccessorySize::L}},
    {3304, {"Tube Line 04"_s, AccessorySize::XL}},
    {3305, {"Tube Line 05"_s, AccessorySize::XL}},
    {3306, {"Tube Line 06"_s, AccessorySize::M}},
    {3307, {"Tube Line 07"_s, AccessorySize::M}},
    {3308, {"Tube Line 08"_s, AccessorySize::M}},
    {3309, {"Tube Line 09"_s, AccessorySize::L}},
    {3310, {"Tube Line 10"_s, AccessorySize::L}},

    {3351, {"Radar Plate 01"_s, AccessorySize::M}},
    {3352, {"Radar Plate 02"_s, AccessorySize::M}},
    {3353, {"Radar Plate 03"_s, AccessorySize::M}},
    {3354, {"Radar Plate 04"_s, AccessorySize::M}},
    {3355, {"Radar Plate 05"_s, AccessorySize::M}},
    {3356, {"Radar Pod 01"_s,   AccessorySize::M}},
    {3357, {"Radar Pod 02"_s,   AccessorySize::M}},
    {3358, {"Radar Pod 03"_s,   AccessorySize::M}},
    {3359, {"Radar Pod 04"_s,   AccessorySize::M}},
    {3360, {"Radar Pod 05"_s,   AccessorySize::M}},

    {3401, {"Tri Pod 01"_s,    AccessorySize::M}},
    {3402, {"Tri Pod 02"_s,    AccessorySize::M}},
    {3403, {"Tri Pod 03"_s,    AccessorySize::M}},
    {3404, {"Tri Pod 04"_s,    AccessorySize::M}},
    {3405, {"Tri Pod 05"_s,    AccessorySize::M}},
    {3406, {"Signal Pod 01"_s, AccessorySize::M}},
    {3407, {"Signal Pod 02"_s, AccessorySize::M}},
    {3408, {"Signal Pod 03"_s, AccessorySize::M}},
    {3409, {"Signal Pod 04"_s, AccessorySize::M}},
    {3410, {"Signal Pod 05"_s, AccessorySize::M}},
    // endregion
};
