#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/Array.h>
#include <Corrade/Containers/StringView.h>

#include <Magnum/Types.h>

using namespace Corrade;
using namespace Containers::Literals;
using namespace Magnum;

struct StoryProgressPoint {
    Int id;
    Containers::StringView chapter;
    Containers::StringView point;
    Containers::StringView after = nullptr;
};

static const Corrade::Containers::Array<StoryProgressPoint> story_progress
{
    InPlaceInit,
    {
        {0x0000, "Chapter 1"_s, "Chapter start (company isn't named yet)"_s},
        {0x0064, "Chapter 1"_s, "First time in the hangar"_s},
        {0x0065, "Chapter 1"_s, "After 1st meeting with Quin in mission section"_s},
        {0x0066, "Chapter 1"_s, "Talking with Reina and Quin in hangar"_s,                    "After training"_s},
        {0x0067, "Chapter 1"_s, "Returned to hangar"_s,                                       "After training"_s},
        {0x0068, "Chapter 1"_s, "Talked with Quin in development section"_s,                  "After training"_s},
        {0x0069, "Chapter 1"_s, "Talked with Waltz in armour section"_s,                      "After training"_s},
        {0x00C8, "Chapter 1"_s, "Talked with Kael in tuning section"_s,                       "After training"_s},
        {0x00C9, "Chapter 1"_s, "Got mission 2 briefing"_s,                                   "After training"_s},
        {0x012C, "Chapter 1"_s, "Talking with Reina"_s,                                       "After mission 2"_s},
        {0x012D, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 2"_s},
        {0x012E, "Chapter 1"_s, "Talked with Kael in tuning section"_s,                       "After mission 2"_s},
        {0x012F, "Chapter 1"_s, "Talked with Reina in hangar"_s,                              "After mission 2"_s},
        {0x0130, "Chapter 1"_s, "Got mission 3 briefing"_s,                                   "After mission 2"_s},
        {0x0190, "Chapter 1"_s, "Talking with Reina"_s,                                       "After mission 3"_s},
        {0x0191, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 3"_s},
        {0x0192, "Chapter 1"_s, "Talked with Waltz in armour section"_s,                      "After mission 3"_s},
        {0x0193, "Chapter 1"_s, "Got mission 4 briefing"_s,                                   "After mission 3"_s},
        {0x01F4, "Chapter 1"_s, "Talking with Reina"_s,                                       "After mission 4"_s},
        {0x01F5, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 4"_s},
        {0x01F6, "Chapter 1"_s, "Talked with Waltz in armour section"_s,                      "After mission 4"_s},
        {0x01F7, "Chapter 1"_s, "Talked with Reina in hangar"_s,                              "After mission 4"_s},
        {0x01F8, "Chapter 1"_s, "Got mission 5 and hunt 1 briefing"_s,                        "After mission 4"_s},
        {0x0258, "Chapter 1"_s, "Meeting Neon and Aine"_s,                                    "After mission 5"_s},
        {0x0259, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 5"_s},
        {0x025A, "Chapter 1"_s, "Got mission 6 briefing"_s,                                   "After mission 5"_s},
        {0x02BC, "Chapter 1"_s, "Talking with Reina"_s,                                       "After mission 6"_s},
        {0x02BD, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 6"_s},
        {0x02BE, "Chapter 1"_s, "Got hunt 2 briefing"_s,                                      "After mission 6"_s},
        {0x02BF, "Chapter 1"_s, "Met Ellenier"_s,                                             "After mission 6"_s},
        {0x02C0, "Chapter 1"_s, "Got mission 7 briefing"_s,                                   "After mission 6"_s},
        {0x0320, "Chapter 1"_s, "Talking with Nier"_s,                                        "After mission 7"_s},
        {0x0321, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 7"_s},
        {0x0322, "Chapter 1"_s, "Talked with Quin, Reina, and Nier in development section"_s, "After mission 7"_s},
        {0x0323, "Chapter 1"_s, "Got mission 8 briefing"_s,                                   "After mission 7"_s},
        {0x0384, "Chapter 1"_s, "Talking with crew in hangar"_s,                              "After mission 8"_s},
        {0x0385, "Chapter 1"_s, "Returned to hangar"_s,                                       "After mission 8"_s},
        {0x0386, "Chapter 1"_s, "Got hunt 3 briefing"_s,                                      "After mission 8"_s},
        {0x0387, "Chapter 1"_s, "Talked with Reina, Nier, and Quin in development section"_s, "After mission 8"_s},

        {0x0388, "Chapter 2"_s, "Chapter start"_s},
        {0x0389, "Chapter 2"_s, "Got mission 9 briefing"_s},
        {0x03E8, "Chapter 2"_s, "Talking with Reina in hangar"_s,           "After mission 9"_s},
        {0x03E9, "Chapter 2"_s, "Returned to hangar"_s,                     "After mission 9"_s},
        {0x03EA, "Chapter 2"_s, "Talked with crew in armour section"_s,     "After mission 9"_s},
        {0x03EB, "Chapter 2"_s, "Got mission 10 briefing"_s,                "After mission 9"_s},
        {0x044C, "Chapter 2"_s, "Talking with Reina in hangar"_s,           "After mission 10"_s},
        {0x044D, "Chapter 2"_s, "Returned to hangar"_s,                     "After mission 10"_s},
        {0x044E, "Chapter 2"_s, "Got mission 11 briefing"_s,                "After mission 10"_s},
        {0x04B0, "Chapter 2"_s, "Talking with Reina and Nier in hangar"_s,  "After mission 11"_s},
        {0x04B1, "Chapter 2"_s, "Returned to hangar"_s,                     "After mission 11"_s},
        {0x04B2, "Chapter 2"_s, "Got mission 12 briefing"_s,                "After mission 11"_s},
        {0x0514, "Chapter 2"_s, "Talking with Reina and Waltz in hangar"_s, "After mission 12"_s},
        {0x0515, "Chapter 2"_s, "Returned to hangar"_s,                     "After mission 12"_s},
        {0x0516, "Chapter 2"_s, "Got hunt 4 and mission 13 briefing"_s,     "After mission 12"_s},

        {0x0578, "Chapter 3"_s, "Chapter start, talking with Reina"_s,        "After mission 13"_s},
        {0x0579, "Chapter 3"_s, "Returned to hangar"_s,                       "After mission 13"_s},
        {0x057A, "Chapter 3"_s, "Talked with Reina in development section"_s, "After mission 13"_s},
        {0x057B, "Chapter 3"_s, "Got briefing for challenges 1, 2, and 3"_s,  "After mission 13"_s},
        {0x057C, "Chapter 3"_s, "Talked with Reina about device"_s,           "After mission 13"_s},
        {0x057D, "Chapter 3"_s, "Got mission 14 briefing"_s,                  "After mission 13"_s},
        {0x05DC, "Chapter 3"_s, "Talking with Reina and Nier"_s,              "After mission 14"_s},
        {0x05DD, "Chapter 3"_s, "Returned to hangar"_s,                       "After mission 14"_s},
        {0x05DE, "Chapter 3"_s, "Got briefing for mission 15 and hunt 5"_s,   "After mission 14"_s},
        {0x0640, "Chapter 3"_s, "Talking with Nier and Kazu, and Reina"_s,    "After mission 15"_s},
        {0x0641, "Chapter 3"_s, "Returned to hangar"_s,                       "After mission 15"_s},
    }
};
