#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/Array.h>
#include <Corrade/Containers/StaticArray.h>
#include <Corrade/Containers/String.h>

#include <Magnum/Magnum.h>
#include <Magnum/Math/Color.h>

#include "WeaponPart.h"
#include "CustomStyle.h"

using namespace Corrade;
using namespace Magnum;

#define c(enumerator, ...) enumerator,
enum class WeaponType {
    #include "../Maps/WeaponTypes.hpp"
};

enum class DamageType {
    #include "../Maps/DamageTypes.hpp"
};

enum class EffectColourMode {
    #include "../Maps/EffectColourModes.hpp"
};
#undef c

struct Weapon {
    Weapon() = default;

    Weapon(const Weapon& other);
    Weapon& operator=(const Weapon& other);

    Weapon(Weapon&& other) = default;
    Weapon& operator=(Weapon&& other) = default;

    Containers::String name;
    WeaponType type = WeaponType::Melee;
    Containers::Array<WeaponPart> parts;
    Containers::StaticArray<16, CustomStyle> customStyles{ValueInit};
    bool attached = false;
    DamageType damageType = DamageType::Physical;
    bool dualWield = false;
    EffectColourMode effectColourMode = EffectColourMode::Default;
    Color4 effectColour{0.0f};
};
