#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <string>

#include <Corrade/Containers/Array.h>
#include <Corrade/Containers/String.h>

#include "../Profile/Profile.h"

using namespace Corrade;

struct Backup {
    Containers::String filename;
    Containers::String company;
    ProfileType type;
    struct {
        int year;
        int month;
        int day;
        int hour;
        int minute;
        int second;
    } timestamp;
    Containers::Array<Containers::String> includedFiles;
};

class ProfileManager {
    public:
        explicit ProfileManager(Containers::StringView save_dir, Containers::StringView backup_dir);

        auto ready() const -> bool;
        auto lastError() -> Containers::StringView;

        auto profiles() -> Containers::ArrayView<Profile>;
        auto refreshProfiles() -> bool;

        auto getProfile(std::size_t index) -> Profile*;
        auto deleteProfile(std::size_t index, bool delete_builds) -> bool;
        auto backupProfile(std::size_t index, bool backup_builds) -> bool;

        auto backups() -> Containers::ArrayView<Backup>;
        void refreshBackups();

        auto deleteBackup(std::size_t index) -> bool;
        auto restoreBackup(std::size_t index) -> bool;

    private:
        bool _ready = false;
        Containers::String _lastError;

        Containers::StringView _saveDirectory;
        Containers::StringView _backupsDirectory;

        Containers::Array<Profile> _profiles;
        Containers::Array<Backup> _backups;
};
