#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/GrowableArray.h>
#include <Corrade/Containers/String.h>
#include <Corrade/Containers/StringView.h>

#include "Serialisers/AbstractUnrealPropertySerialiser.h"
#include "Serialisers/AbstractUnrealCollectionPropertySerialiser.h"

#include "Types/UnrealPropertyBase.h"

using namespace Corrade;

class BinaryReader;
class BinaryWriter;

class PropertySerialiser {
    public:
        static auto instance() -> PropertySerialiser&;

        auto read(BinaryReader& reader) -> UnrealPropertyBase::ptr;
        auto readItem(BinaryReader& reader, Containers::String type, UnsignedLong value_length,
                      Containers::String name) -> UnrealPropertyBase::ptr;
        auto readSet(BinaryReader& reader, Containers::StringView item_type, UnsignedInt count) -> Containers::Array<UnrealPropertyBase::ptr>;
        auto deserialise(Containers::String name, Containers::String type, UnsignedLong value_length,
                         BinaryReader& reader) -> UnrealPropertyBase::ptr;

        auto serialise(UnrealPropertyBase::ptr& prop, Containers::StringView item_type, UnsignedLong& bytes_written,
                       BinaryWriter& writer) -> bool;
        auto write(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written, BinaryWriter& writer) -> bool;
        auto writeItem(UnrealPropertyBase::ptr& prop, Containers::StringView item_type, UnsignedLong& bytes_written,
                       BinaryWriter& writer) -> bool;
        auto writeSet(Containers::ArrayView<UnrealPropertyBase::ptr> props, Containers::StringView item_type,
                      UnsignedLong& bytes_written, BinaryWriter& writer) -> bool;

    private:
        PropertySerialiser();

        auto getSerialiser(Containers::StringView item_type) -> AbstractUnrealPropertySerialiser*;
        auto getCollectionSerialiser(Containers::StringView item_type) -> AbstractUnrealCollectionPropertySerialiser*;

        Containers::Array<AbstractUnrealPropertySerialiser::ptr> _serialisers;
        Containers::Array<AbstractUnrealCollectionPropertySerialiser::ptr> _collectionSerialisers;
};
