#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/Array.h>
#include <Corrade/Containers/ArrayView.h>
#include <Corrade/Containers/Pointer.h>
#include <Corrade/Containers/StringView.h>

#include <Magnum/Types.h>

#include "../Types/UnrealPropertyBase.h"

using namespace Corrade;
using namespace Magnum;

class BinaryReader;
class BinaryWriter;
class PropertySerialiser;

class AbstractUnrealCollectionPropertySerialiser {
    public:
        using ptr = Containers::Pointer<AbstractUnrealCollectionPropertySerialiser>;

        virtual ~AbstractUnrealCollectionPropertySerialiser() = default;

        virtual auto types() -> Containers::ArrayView<const Containers::String> = 0;

        virtual auto deserialise(Containers::StringView name, Containers::StringView type,
                                 UnsignedLong value_length, UnsignedInt count, BinaryReader& reader,
                                 PropertySerialiser& serialiser) -> Containers::Array<UnrealPropertyBase::ptr> = 0;

        virtual auto serialise(Containers::ArrayView<UnrealPropertyBase::ptr> props, Containers::StringView item_type,
                               UnsignedLong& bytes_written, BinaryWriter& writer, PropertySerialiser& serialiser) -> bool = 0;
};
