// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../../Logger/Logger.h"

#include "BytePropertySerialiser.h"

auto BytePropertySerialiser::types() -> Containers::ArrayView<const Containers::String> {
    using namespace Containers::Literals;
    static const Containers::Array<Containers::String> types{InPlaceInit, {"ByteProperty"_s}};
    return types;
}

auto BytePropertySerialiser::deserialise(Containers::StringView name, Containers::StringView type,
                                         UnsignedLong value_length, BinaryReader& reader,
                                         PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<ByteProperty>();

    if(value_length != UnsignedLong(-1)) {
        if(!reader.readUEString(prop->enumType)) {
            LOG_ERROR_FORMAT("Couldn't read byte property {}'s enum type.", name);
            return nullptr;
        }

        char terminator;
        if(!reader.readChar(terminator) || terminator != '\0') {
            LOG_ERROR_FORMAT("Couldn't read a null byte in byte property {}.", name);
            return nullptr;
        }
    }

    if(!reader.readUEString(prop->enumValue)) {
        LOG_ERROR("Couldn't read byte property's enum value.");
        return nullptr;
    }

    prop->valueLength = value_length;

    //UnsignedInt count = 0;
    //if(!reader.readUnsignedInt(count)) {
    //    return nullptr;
    //}

    //if(!reader.readArray(prop->value, count)) {
    //    return nullptr;
    //}

    return prop;
}

auto BytePropertySerialiser::serialise(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                       BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto byte_prop = dynamic_cast<ByteProperty*>(prop.get());
    if(!byte_prop) {
        LOG_ERROR("The property is not a valid byte property.");
        return false;
    }

    //writer.writeValueToArray<char>('\0');
    //bytes_written += writer.writeValueToArray<UnsignedInt>(byte_prop->value.size());
    //bytes_written += writer.writeDataToArray<char>(byte_prop->value);

    if(byte_prop->valueLength != UnsignedLong(-1)) {
        writer.writeUEStringToArray(byte_prop->enumType);
        writer.writeValueToArray<char>('\0');
    }

    bytes_written += writer.writeUEStringToArray(byte_prop->enumValue);

    return true;
}
