// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../../Logger/Logger.h"

#include "DateTimePropertySerialiser.h"

auto DateTimePropertySerialiser::deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                                     UnsignedLong value_length, BinaryReader& reader,
                                                     PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<DateTimeStructProperty>();

    if(!reader.readUnsignedLong(prop->timestamp)) {
        LOG_ERROR_FORMAT("Couldn't read date/time property {}'s value.", name);
        return nullptr;
    }

    return prop;
}

auto DateTimePropertySerialiser::serialiseProperty(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                                   BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto dt_prop = dynamic_cast<DateTimeStructProperty*>(prop.get());
    if(!dt_prop) {
        LOG_ERROR("The property is not a valid date/time property.");
        return false;
    }

    bytes_written += writer.writeValueToArray<UnsignedLong>(dt_prop->timestamp);

    return true;
}
