// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../../Logger/Logger.h"

#include "GuidPropertySerialiser.h"

using namespace Containers::Literals;

auto GuidPropertySerialiser::deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                                 UnsignedLong value_length, BinaryReader& reader,
                                                 PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<GuidStructProperty>();

    if(!reader.readStaticArray(prop->guid)) {
        LOG_ERROR_FORMAT("Couldn't read GUID property {}'s value.", name);
        return nullptr;
    }

    return prop;
}

auto GuidPropertySerialiser::serialiseProperty(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                               BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto guid_prop = dynamic_cast<GuidStructProperty*>(prop.get());
    if(!guid_prop) {
        LOG_ERROR("The property is not a valid byte property.");
        return false;
    }

    bytes_written += writer.writeDataToArray<char>({guid_prop->guid.data(), guid_prop->guid.size()});

    return true;
}
