// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../../Logger/Logger.h"

#include "IntPropertySerialiser.h"

auto IntPropertySerialiser::deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                                UnsignedLong value_length, BinaryReader& reader,
                                                PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<IntProperty>();

    if(value_length == UnsignedLong(-1)) {
        if(!reader.readInt(prop->value)) {
            LOG_ERROR("Couldn't read int property's value.");
            return nullptr;
        }

        prop->valueLength = UnsignedLong(-1);
        return prop;
    }

    char terminator;
    if(!reader.readChar(terminator) || terminator != '\0') {
        LOG_ERROR_FORMAT("Couldn't read a null byte in int property {}.", name);
        return nullptr;
    }

    if(!reader.readInt(prop->value)) {
        LOG_ERROR_FORMAT("Couldn't read int property {}'s value.", name);
        return nullptr;
    }

    prop->name.emplace(name);

    return prop;
}

auto IntPropertySerialiser::serialiseProperty(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                              BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto int_prop = dynamic_cast<IntProperty*>(prop.get());
    if(!int_prop) {
        LOG_ERROR("The property is not a valid int property.");
        return false;
    }

    if(prop->valueLength != UnsignedLong(-1)) {
        writer.writeValueToArray<char>('\0');
    }

    bytes_written += writer.writeValueToArray<Int>(int_prop->value);

    return true;
}
