// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../PropertySerialiser.h"
#include "../Types/IntProperty.h"
#include "../Types/NoneProperty.h"
#include "../../Logger/Logger.h"

#include "ResourcePropertySerialiser.h"

using namespace Containers::Literals;

auto ResourcePropertySerialiser::deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                                     UnsignedLong value_length, BinaryReader& reader,
                                                     PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<ResourceItemValue>();

    auto id_prop = serialiser.read(reader);
    if(!id_prop) {
        LOG_ERROR("Couldn't read the ID property."_s);
        return nullptr;
    }

    if((*id_prop->name) != "ID_4_AAE08F17428E229EC7A2209F51081A21"_s ||
       id_prop->propertyType != "IntProperty"_s                      ||
       dynamic_cast<IntProperty*>(id_prop.get()) == nullptr)
    {
        LOG_ERROR("The ID property is invalid."_s);
        return nullptr;
    }

    prop->id = dynamic_cast<IntProperty*>(id_prop.get())->value;

    auto value_prop = serialiser.read(reader);
    if(!value_prop) {
        LOG_ERROR("Couldn't read the value property."_s);
        return nullptr;
    }

    if((*value_prop->name) != "Quantity_3_560F09B5485C365D3041888910019CE3"_s ||
       value_prop->propertyType != "IntProperty"_s                            ||
       dynamic_cast<IntProperty*>(value_prop.get()) == nullptr)
    {
        LOG_ERROR("The value property is invalid."_s);
        return nullptr;
    }

    prop->quantity = dynamic_cast<IntProperty*>(value_prop.get())->value;

    auto none_prop = serialiser.read(reader);

    if(!none_prop                                  ||
       (*none_prop->name) != "None"_s              ||
       none_prop->propertyType != "NoneProperty"_s ||
       !dynamic_cast<NoneProperty*>(none_prop.get()))
    {
        LOG_ERROR("Couldn't find a terminating NoneProperty."_s);
        return nullptr;
    }

    return prop;
}

auto ResourcePropertySerialiser::serialiseProperty(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                                   BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto res_prop = dynamic_cast<ResourceItemValue*>(prop.get());
    if(!res_prop) {
        LOG_ERROR("The property is not a valid ResourceItemValue property.");
        return false;
    }

    bytes_written += writer.writeUEStringToArray("ID_4_AAE08F17428E229EC7A2209F51081A21"_s) +
                     writer.writeUEStringToArray("IntProperty"_s) +
                     writer.writeValueToArray<UnsignedLong>(4ull) +
                     writer.writeValueToArray<char>('\0') +
                     writer.writeValueToArray<Int>(res_prop->id);

    bytes_written += writer.writeUEStringToArray("Quantity_3_560F09B5485C365D3041888910019CE3"_s) +
                     writer.writeUEStringToArray("IntProperty"_s) +
                     writer.writeValueToArray<UnsignedLong>(4ull) +
                     writer.writeValueToArray<char>('\0') +
                     writer.writeValueToArray<Int>(res_prop->quantity);

    bytes_written += writer.writeUEStringToArray("None"_s);

    return true;
}
