// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../../Logger/Logger.h"

#include "StringPropertySerialiser.h"

auto StringPropertySerialiser::types() -> Containers::ArrayView<const Containers::String> {
    using namespace Containers::Literals;
    static const Containers::Array<Containers::String> types{InPlaceInit,
                                                             {"NameProperty"_s, "StrProperty"_s,
                                                              "SoftObjectProperty"_s, "ObjectProperty"_s}};
    return types;
}

auto StringPropertySerialiser::deserialise(Containers::StringView name, Containers::StringView type,
                                           UnsignedLong value_length, BinaryReader& reader,
                                           PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<StringProperty>(type);

    if(value_length != UnsignedLong(-1)) {
        char terminator;
        if(!reader.readChar(terminator) || terminator != '\0') {
            LOG_ERROR_FORMAT("Couldn't read a null byte in string property {}.", name);
            return nullptr;
        }
    }

    if(!reader.readUEString(prop->value)) {
        LOG_ERROR_FORMAT("Couldn't read string property {}'s value.", name);
        return nullptr;
    }

    prop->valueLength = value_length;

    return prop;
}

auto StringPropertySerialiser::serialise(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                         BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto str_prop = dynamic_cast<StringProperty*>(prop.get());
    if(!str_prop) {
        LOG_ERROR("The property is not a valid string property.");
        return false;
    }

    if(str_prop->valueLength != UnsignedLong(-1)) {
        writer.writeValueToArray<char>('\0');
    }

    bytes_written += writer.writeUEStringToArray(str_prop->value);

    return true;
}
