#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/ArrayView.h>
#include <Corrade/Containers/StringView.h>

#include "AbstractUnrealCollectionPropertySerialiser.h"
#include "AbstractUnrealPropertySerialiser.h"
#include "AbstractUnrealStructSerialiser.h"

#include "../Types/StructProperty.h"

class StructSerialiser : public AbstractUnrealPropertySerialiser, public AbstractUnrealCollectionPropertySerialiser {
    public:
        using ptr = Containers::Pointer<StructSerialiser>;

        auto types() -> Containers::ArrayView<const Containers::String> override;

        auto deserialise(Containers::StringView name, Containers::StringView type, UnsignedLong value_length,
                         UnsignedInt count, BinaryReader& reader, PropertySerialiser& serialiser) -> Containers::Array<UnrealPropertyBase::ptr> override;
        auto deserialise(Containers::StringView name, Containers::StringView type, UnsignedLong value_length,
                         BinaryReader& reader, PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr override;

        auto serialise(Containers::ArrayView<UnrealPropertyBase::ptr> props, Containers::StringView item_type,
                       UnsignedLong& bytes_written, BinaryWriter& writer, PropertySerialiser& serialiser) -> bool override;
        auto serialise(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                       BinaryWriter& writer, PropertySerialiser& serialiser) -> bool override;

    private:
        auto readStructValue(Containers::StringView name, Containers::StringView type, UnsignedLong value_length,
                             BinaryReader& reader, PropertySerialiser& serialiser) -> StructProperty::ptr;
        auto writeStructValue(StructProperty* prop, UnsignedLong& bytes_written, BinaryWriter& writer, PropertySerialiser& serialiser) -> bool;
};
