// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/String.h>

#include "../BinaryReader.h"
#include "../BinaryWriter.h"

#include "TextPropertySerialiser.h"

auto TextPropertySerialiser::deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                                 UnsignedLong value_length, BinaryReader& reader,
                                                 PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<TextProperty>();

    Long start_position = reader.position();

    char terminator;
    if(!reader.readChar(terminator) || terminator != '\0') {
        return nullptr;
    }

    if(reader.peekChar() > 0) {
        if(!reader.readArray(prop->flags, 8)) {
            return nullptr;
        }
    }
    else {
        if(!reader.readArray(prop->flags, 4)) {
            return nullptr;
        }
    }

    if(!reader.readChar(prop->id)) {
        return nullptr;
    }

    auto interval = reader.position() - start_position;

    do {
        Containers::String str;

        if(!reader.readUEString(str)) {
            return nullptr;
        }

        arrayAppend(prop->data, std::move(str));

        interval = reader.position() - start_position;
    } while(UnsignedLong(interval) < value_length);

    prop->value = prop->data.back();

    return prop;
}

auto TextPropertySerialiser::serialiseProperty(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                               BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto text_prop = dynamic_cast<TextProperty*>(prop.get());

    if(!text_prop) {
        return false;
    }

    writer.writeValueToArray<char>('\0');
    bytes_written += writer.writeDataToArray<char>(text_prop->flags);
    for(const auto& str : text_prop->data) {
        bytes_written += writer.writeUEStringToArray(str);
    }

    return true;
}
