#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <type_traits>

#include <Corrade/Containers/Array.h>
#include <Corrade/Containers/ArrayView.h>
#include <Corrade/Containers/StringView.h>

#include "AbstractUnrealPropertySerialiser.h"
#include "../Types/StructProperty.h"

template<typename T>
class UnrealPropertySerialiser : public AbstractUnrealPropertySerialiser {
        static_assert(std::is_base_of<UnrealPropertyBase, T>::value, "T must be derived from UnrealPropertyBase.");

    public:
        using ptr = Containers::Pointer<UnrealPropertySerialiser<T>>;

        auto types() -> Containers::ArrayView<const Containers::String> override {
            static const Containers::Array<Containers::String> types = []{
                Containers::Array<Containers::String> array;
                Containers::Pointer<T> p(new T);
                if(std::is_base_of<StructProperty, T>::value) {
                    array = Containers::Array<Containers::String>{InPlaceInit, {dynamic_cast<StructProperty*>(p.get())->structType}};
                }
                else {
                    array = Containers::Array<Containers::String>{InPlaceInit, {p->propertyType}};
                }
                return array;
            }();
            return types;
        }

        auto deserialise(Containers::StringView name, Containers::StringView type, UnsignedLong value_length,
                         BinaryReader& reader, PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr override
        {
            return deserialiseProperty(name, type, value_length, reader, serialiser);
        }

        auto serialise(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written, BinaryWriter& writer,
                       PropertySerialiser& serialiser) -> bool override
        {
            return serialiseProperty(prop, bytes_written, writer, serialiser);
        }

    private:
        virtual auto deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                         UnsignedLong value_length, BinaryReader& reader,
                                         PropertySerialiser& serialiser) -> typename UnrealPropertyBase::ptr = 0;

        virtual auto serialiseProperty(typename UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                       BinaryWriter& writer, PropertySerialiser& serialiser) -> bool = 0;
};
