// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "../BinaryReader.h"
#include "../BinaryWriter.h"
#include "../../Logger/Logger.h"

#include "Vector2DPropertySerialiser.h"

auto Vector2DPropertySerialiser::deserialiseProperty(Containers::StringView name, Containers::StringView type,
                                                     UnsignedLong value_length, BinaryReader& reader,
                                                     PropertySerialiser& serialiser) -> UnrealPropertyBase::ptr
{
    auto prop = Containers::pointer<Vector2DStructProperty>();

    if(!reader.readFloat(prop->x) || !reader.readFloat(prop->y)) {
        LOG_ERROR_FORMAT("Couldn't read 2D vector property {}'s value.", name);
        return nullptr;
    }

    return prop;
}

auto Vector2DPropertySerialiser::serialiseProperty(UnrealPropertyBase::ptr& prop, UnsignedLong& bytes_written,
                                                   BinaryWriter& writer, PropertySerialiser& serialiser) -> bool
{
    auto vector = dynamic_cast<Vector2DStructProperty*>(prop.get());
    if(!vector) {
        LOG_ERROR("The property is not a valid 2D vector property.");
        return false;
    }

    bytes_written += writer.writeValueToArray<Float>(vector->x) + writer.writeValueToArray<Float>(vector->y);

    return true;
}
