#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <Corrade/Containers/GrowableArray.h>
#include <Corrade/Containers/Pointer.h>
#include <Corrade/Containers/String.h>
#include <Corrade/Containers/StringView.h>

#include "UnrealPropertyBase.h"

struct ArrayProperty : public UnrealPropertyBase {
    using ptr = Containers::Pointer<ArrayProperty>;

    ArrayProperty() {
        using namespace Containers::Literals;
        propertyType = "ArrayProperty"_s;
    }

    template<typename T>
    std::enable_if_t<std::is_base_of<UnrealPropertyBase, T>::value, T*>
    at(std::size_t index) {
        if(index >= items.size()) {
            return nullptr;
        }

        return static_cast<T*>(items[index].get());
    }

    Containers::String itemType;
    Containers::Array<UnrealPropertyBase::ptr> items;
};
