#pragma once

// MassBuilderSaveTool
// Copyright (C) 2021-2022 Guillaume Jacquemin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <type_traits>

#include <Corrade/Containers/GrowableArray.h>
#include <Corrade/Containers/Pointer.h>
#include <Corrade/Containers/StringView.h>

#include "StructProperty.h"

using namespace Corrade;

struct GenericStructProperty : public StructProperty {
    using ptr = Containers::Pointer<GenericStructProperty>;

    template<typename T>
    std::enable_if_t<std::is_base_of<UnrealPropertyBase, T>::value, T*>
    at(Containers::StringView name) {
        for(auto& item : properties) {
            if(item->name == name) {
                return static_cast<T*>(item.get());
            }
        }
        return nullptr;
    }

    template<typename T>
    std::enable_if_t<std::is_base_of<UnrealPropertyBase, T>::value, typename T::ptr>
    atMove(Containers::StringView name) {
        for(auto& item : properties) {
            if(item && item->name == name) {
                return Containers::Pointer<T>{static_cast<T*>(item.release())};
            }
        }
        return nullptr;
    }

    Containers::Array<UnrealPropertyBase::ptr> properties;
};
